from gmat.remma.random_pair import random_pair, random_pairAD
from gmat.remma.annotation import gtf_to_gene_info, annotation_snp_pos, annotation_snp_nearest_gene
from gmat.remma.pre_matrix import pre_design_matrix



import logging
import sys
import numpy as np
import pandas as pd
from scipy.sparse import csr_matrix


def pre_design_matrix(pheno_file, bed_file):
    """
    准备表型向量、固定效应设计矩阵、随机效应设计矩阵
    :param pheno_file:
    :param bed_file:
    :return:y, xmat, zmat
    """
    id_bed_lst = []
    fin = open(bed_file + '.fam')
    for line in fin:
        arr = line.split()
        id_bed_lst.append(" ".join([arr[0], arr[1]]))
    fin.close()
    id_pheno_lst = {}
    fin = open(pheno_file)
    for line in fin:
        arr = line.split()
        try:
            id_pheno_lst[" ".join([arr[0], arr[1]])].append(" ".join(arr))
        except Exception as e:
            del e
            id_pheno_lst[" ".join([arr[0], arr[1]])] = [" ".join(arr)]
    fin.close()
    id_not_pheno = set(id_bed_lst) - set(list(id_pheno_lst.keys()))
    if len(id_not_pheno) > 0:
        logging.error('The below genotyped id is not in the phenotype file:\n {}'.format('\n'.join(list(id_not_pheno))))
        sys.exit()
    fout = open(pheno_file + '.temp', 'w')
    for id in id_bed_lst:
        for val in id_pheno_lst[id]:
            fout.write(val + '\n')
    fout.close()
    df = pd.read_csv(pheno_file + '.temp', header=None, sep='\s+')
    y = np.array(df.iloc[:, -1:])
    xmat = np.array(df.iloc[:, 2:-1])
    id_dct = {}
    row = []
    col = []
    j = 0
    for i in range(df.shape[0]):
        row.append(i)
        if df.iloc[i, 1] not in id_dct:
            id_dct[df.iloc[i, 1]] = j
            j += 1
        col.append(id_dct[df.iloc[i, 1]])
    zmat = csr_matrix(([1.0]*len(row), (row, col)))
    return y, xmat, zmat



import numpy as np
from pandas_plink import read_plink1_bin



def read_plink(bed_file):
    snp_info = read_plink1_bin(bed_file + ".bed", bed_file + ".bim", bed_file + ".fam", verbose=False)
    return snp_info.values


def impute_geno(snp_mat):
    ind = np.where(np.isnan(snp_mat))
    ind_len = len(ind[0])
    snp_mat.val[ind] = np.random.choice([0.0, 1.0, 2.0], ind_len)
    return snp_mat


def shuffle_bed(bed_file):
    try:
        from pysnptools.snpreader import Bed
    except Exception as e:
        print(e)
        exit()
    data = Bed(bed_file, count_A1=False).read()
    num_snp = data.val.shape[1]
    for i in range(num_snp):
        np.random.shuffle(data.val[:, i])
    Bed.write(bed_file + '_shuffle', data, count_A1=False)
